//variables globales qui définissent l'état du jeu
let nombreCache;          // le nombre caché à trouver
let nbreCoupsJoues = 0;   // le nombre de coups que l'utilisateur a joué
const NBRE_COUPS_MAX = 10;    // le nombre de coups maximum que l'utilisateur peut jouer

/**
 * retourne un nombre entier tiré au hasard dans un intervalle (les bornes de l'intervalle
 * sont incluses).
 * pour en savoir plus :
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/random
 * @param {Number} min la borne inférieure de l'intervalle.
 * @param {Number} max la borne supérieure de l'intervalle.
 * @returns {Number} le nombre tiré au hasard.
 */
function getRandomInt(min, max) {
    min = Math.ceil(min);   // au cas ou min et max ne seraient pas des entiers
    max = Math.floor(max);
    return Math.floor(Math.random() * (max + 1 - min)) + min;
}

/**
 * Action effectuée lorsque l'utilisateur débute une partie.
 * Tire un nombre au hasard, remet à zéro le nombre de coups joués et 
 * remet à jour la page HTML .
 * Cette action est appelée une première fois au chargement de la page 
 * 
 * Ensuite elle est appelée à chaque fois que le joueur termine une partie et
 * en rédébute une en cliquant sur le bouton "Nouvelle Partie" (attribut
 * onclick de l'élément <button> d'id 'debutPartieBtn').
 */
function debuterPartie() {
    // récupération des bornes de l'intervalle dans lequel se trouve le nombre caché
    // définies par les attributs min et max de l'input "proposition"
    const borneMin = document.getElementById("proposition").min;
    const borneMax = document.getElementById("proposition").max;
    // génération aléatoire du nombre caché
    nombreCache = getRandomInt(borneMin, borneMax);
    // réinitialisation du compteur du nombre de coups joués
    nbreCoupsJoues = 0;
    // réinitialisation de la page HTML
    // cacher le bouton 'rejouer' et montrer les boutons 'jouer' et 'abandonner' 
    // en modifiant les styles CSS des éléments (<div>) correspondants.
    document.getElementById("partieEncours").style.display = 'block';
    document.getElementById("partieTerminee").style.display = 'none';
    document.getElementById("message").innerHTML = "<br><br>";
    document.getElementById("proposition").value = "";
}

/**
 * Action effectuée lorsque l'utilisateur soumet une proposition.
 * Cette action compare la valeur proposée avec le nombre caché et met
 * à jour le DOM de la page HTML en fonction de la manière dont
 * cette valeur se situe par rapport au nombre caché.
 */
function jouer() {
    let message = ""; // le message qui sera affiché dans l'élément <div id="message">     
    let valeurProposee;  // la valeur proposée par l'utilisateur
    let partieTerminee = false; // teste si la partie est terminée, sera vraie si
    // l'utilisateur a trouvé le bon nombre, ou si il a épuisé
    // son nombre de coups maximum.

    if (!document.getElementById("proposition").checkValidity()) {
        // la valeur proposée n'est pas un nombre ou n'est pas dans l'intervalle fixé
        // un message d'erreur est affiché (la valeurProposee  n'est pas comptée, le 
        // nbre de coups joués n'est pas modifié).
        document.getElementById("message").className = "perdu";
        message = "valeur incorrecte !!<br><br>";
    } else {
        // la valeur proposée est dans l'intervalle
        valeurProposee = parseInt(document.getElementById("proposition").value); // récupération de la valeur sous la forme d'un nombre entier
        nbreCoupsJoues++;
 
        if (valeurProposee === nombreCache) {
            document.getElementById("message").className = "gagne";
            message = "Bravo ! Vous avez gagné<br>en " + nbreCoupsJoues 
                    + " essai" + plurielSi(nbreCoupsJoues > 1) + " !!";
            partieTerminee = true;
        } else {
            document.getElementById("message").className = "perdu";
            if (valeurProposee < nombreCache) {
                message = "trop petit<br>";
            } else {
                message = "trop grand<br>";
            }
            if (nbreCoupsJoues < NBRE_COUPS_MAX) {
                message += "Rejouez ! il vous reste " + (NBRE_COUPS_MAX - nbreCoupsJoues) 
                              + " essai" + plurielSi((NBRE_COUPS_MAX - nbreCoupsJoues) > 1) ;
            } else {
                message += " - vous avez perdu, le nombre était " + nombreCache;
                partieTerminee = true;
            }
        }
    }
    // afficher le message
    document.getElementById("message").innerHTML = message;
    if (partieTerminee) {
        afficherBoutonRejouer();
    }
}

/**
 * Action effectuée lorque l'utilisateur clique sur le bouton 'abandonner'.
 * Le nombre caché est révélé et le bouton qui permettra de relancer une
 * nouvelle partie est affiché.
 */
function abandonner() {
    const message = "Pourquoi abandonner?<br>Le nombre caché était : " + nombreCache;
    document.getElementById("message").innerHTML = message;
    afficherBoutonRejouer();
}

/**
 * En fin de partie, cache les boutons 'jouer' et 'abandonner' et montre le 
 * bouton 'Nouvelle Partie'  en modifiant les styles CSS des éléments (<div>) correspondants.
 */
function afficherBoutonRejouer() {
    document.getElementById("partieEncours").style.display = 'none';
    document.getElementById("partieTerminee").style.display = 'block';
}

/**
 * fonction utilitaire pour mettre au pluriel ou non une chaîne de caractères
 * @param {boolean} condition la condition pour que la chaîne soit au pluriel
 * @returns {String} 's' si condition vaut true, '' si condition vaut false
 */
 function plurielSi(condition) {
    if (condition) {
         return "s";
    } 
    return "";  // le else est inutile car il y a un return dans le if
}

//----------------------------------------------------
// programme principal
// instructions exécutées lorsque le script est chargé
//----------------------------------------------------

// associe les actions de jeux aux différents boutons 
document.getElementById("jouerBtn").addEventListener("click", jouer);
document.getElementById("abandonnerBtn").addEventListener("click", abandonner);
document.getElementById("debutPartieBtn").addEventListener("click", debuterPartie);

debuterPartie();  // pour initialiser la première partie

